# -*- coding: utf-8 -*-
"""
tests.functional.test_models
~~~~~~~~~~~~~~~~~~~~~~~~~~~~

This module defines models tests.

:copyright: (c) 2017 European Spallation Source ERIC
:license: BSD 2-Clause, see LICENSE for more details.

"""
import ipaddress
import pytest
from wtforms import ValidationError
from app import models


def test_user_groups(user_factory):
    user = user_factory()
    assert user.groups == []
    groups = ["foo", "Another group"]
    user = user_factory(groups=groups)
    assert user.groups == groups


def test_user_is_admin(user_factory):
    user = user_factory(groups=["foo", "CSEntry User"])
    assert not user.is_admin
    user = user_factory(groups=["foo", "CSEntry Admin"])
    assert user.is_admin


def test_user_is_member_of_one_group(user_factory):
    user = user_factory(groups=["one", "two"])
    assert not user.is_member_of_one_group(["create", "admin"])
    user = user_factory(groups=["one", "CSEntry Consultant"])
    assert user.is_member_of_one_group(["create"])
    assert user.is_member_of_one_group(["create", "admin"])
    assert not user.is_member_of_one_group(["admin"])
    user = user_factory(groups=["one", "CSEntry Admin"])
    assert not user.is_member_of_one_group(["create"])
    assert user.is_member_of_one_group(["create", "admin"])
    assert user.is_member_of_one_group(["admin"])


def test_network_ip_properties(network_factory):
    # Create some networks
    network1 = network_factory(
        address="172.16.1.0/24", first_ip="172.16.1.10", last_ip="172.16.1.250"
    )
    network2 = network_factory(
        address="172.16.20.0/26", first_ip="172.16.20.11", last_ip="172.16.20.14"
    )

    assert network1.network_ip == ipaddress.ip_network("172.16.1.0/24")
    assert network1.first == ipaddress.ip_address("172.16.1.10")
    assert network1.last == ipaddress.ip_address("172.16.1.250")
    assert len(network1.ip_range()) == 241
    assert network1.ip_range() == [
        ipaddress.ip_address(f"172.16.1.{i}") for i in range(10, 251)
    ]
    assert network1.ip_range() == network1.available_ips()
    assert network1.used_ips() == []

    assert network2.network_ip == ipaddress.ip_network("172.16.20.0/26")
    assert network2.first == ipaddress.ip_address("172.16.20.11")
    assert network2.last == ipaddress.ip_address("172.16.20.14")
    assert len(network2.ip_range()) == 4
    assert network2.ip_range() == [
        ipaddress.ip_address(f"172.16.20.{i}") for i in range(11, 15)
    ]
    assert network2.ip_range() == network2.available_ips()
    assert network2.used_ips() == []


def test_network_available_and_used_ips(network_factory, interface_factory):
    # Create some networks and interfaces
    network1 = network_factory(
        address="172.16.1.0/24", first_ip="172.16.1.10", last_ip="172.16.1.250"
    )
    network2 = network_factory(
        address="172.16.20.0/26", first_ip="172.16.20.11", last_ip="172.16.20.14"
    )
    for i in range(10, 20):
        interface_factory(network=network1, ip=f"172.16.1.{i}")
    interface_factory(network=network2, ip="172.16.20.13")
    # Check available and used IPs
    assert network1.used_ips() == [
        ipaddress.ip_address(f"172.16.1.{i}") for i in range(10, 20)
    ]
    assert network1.available_ips() == [
        ipaddress.ip_address(f"172.16.1.{i}") for i in range(20, 251)
    ]
    assert network2.used_ips() == [ipaddress.ip_address("172.16.20.13")]
    assert network2.available_ips() == [
        ipaddress.ip_address("172.16.20.11"),
        ipaddress.ip_address("172.16.20.12"),
        ipaddress.ip_address("172.16.20.14"),
    ]

    # Add more interfaces
    interface_factory(network=network2, ip="172.16.20.11")
    interface_factory(network=network2, ip="172.16.20.14")
    assert len(network2.used_ips()) == 3
    assert network2.used_ips() == [
        ipaddress.ip_address("172.16.20.11"),
        ipaddress.ip_address("172.16.20.13"),
        ipaddress.ip_address("172.16.20.14"),
    ]
    assert network2.available_ips() == [ipaddress.ip_address("172.16.20.12")]

    # Add last available IP
    interface_factory(network=network2, ip="172.16.20.12")
    assert network2.used_ips() == [
        ipaddress.ip_address(f"172.16.20.{i}") for i in range(11, 15)
    ]
    assert list(network2.available_ips()) == []


def test_network_gateway(network_factory):
    network = network_factory(address="192.168.0.0/24")
    assert str(network.gateway) == "192.168.0.254"
    network = network_factory(address="172.16.110.0/23")
    assert str(network.gateway) == "172.16.111.254"


def test_mac_address_validation(mac_factory):
    mac = mac_factory(address="F4:A7:39:15:DA:01")
    assert mac.address == "f4:a7:39:15:da:01"
    mac = mac_factory(address="F4-A7-39-15-DA-02")
    assert mac.address == "f4:a7:39:15:da:02"
    mac = mac_factory(address="F4A73915DA06")
    assert mac.address == "f4:a7:39:15:da:06"
    with pytest.raises(ValidationError) as excinfo:
        mac = mac_factory(address="F4A73915DA")
    assert "'F4A73915DA' does not appear to be a MAC address" in str(excinfo.value)


def test_manufacturer_favorite_users(user_factory, manufacturer_factory):
    user1 = user_factory()
    user2 = user_factory()
    user3 = user_factory()
    manufacturer1 = manufacturer_factory()
    manufacturer2 = manufacturer_factory()
    user1.favorite_manufacturers.append(manufacturer2)
    assert manufacturer1.favorite_users == []
    assert manufacturer2.favorite_users == [user1]
    user2.favorite_manufacturers.append(manufacturer2)
    user3.favorite_manufacturers.append(manufacturer1)
    assert manufacturer2.favorite_users == [user1, user2]
    assert user2 in manufacturer2.favorite_users
    assert user2 not in manufacturer1.favorite_users
    assert user3 in manufacturer1.favorite_users


def test_device_type_validation(device_type_factory):
    device_type = device_type_factory(name="PhysicalMachine")
    assert device_type.name == "PhysicalMachine"
    with pytest.raises(ValidationError) as excinfo:
        device_type = device_type_factory(name="Physical Machine")
    assert "'Physical Machine' is an invalid device type name" in str(excinfo.value)


def test_tag_validation(tag_factory):
    tag = tag_factory(name="IOC")
    assert tag.name == "IOC"
    with pytest.raises(ValidationError) as excinfo:
        tag = tag_factory(name="My tag")
    assert "'My tag' is an invalid tag name" in str(excinfo.value)


def test_ansible_groups(ansible_group_factory, host_factory):
    group1 = ansible_group_factory()
    group2 = ansible_group_factory()
    host1 = host_factory(ansible_groups=[group1])
    assert host1.ansible_groups == [group1]
    assert group1.hosts == [host1]
    host1.ansible_groups.append(group2)
    assert host1.ansible_groups == [group1, group2]
    assert group2.hosts == [host1]


def test_ansible_group_is_dynamic(ansible_group_factory):
    group1 = ansible_group_factory()
    assert group1.type == models.AnsibleGroupType.STATIC
    assert not group1.is_dynamic
    group2 = ansible_group_factory(type=models.AnsibleGroupType.NETWORK_SCOPE)
    assert group2.is_dynamic
    group3 = ansible_group_factory(type=models.AnsibleGroupType.NETWORK)
    assert group3.is_dynamic


def test_ansible_groups_children(ansible_group_factory, host_factory):
    group1 = ansible_group_factory()
    group2 = ansible_group_factory()
    group3 = ansible_group_factory()
    group1.children.append(group2)
    group1.children.append(group3)
    assert group1.children == [group2, group3]
    assert group2.parents == [group1]
    assert group3.parents == [group1]
    group4 = ansible_group_factory(parents=[group1])
    assert group1.children == [group2, group3, group4]


def test_host_model(model_factory, item_factory, host_factory):
    host1 = host_factory()
    model1 = model_factory(name="EX3400")
    item_factory(model=model1, host_id=host1.id)
    assert host1.model == "EX3400"


def test_ansible_dynamic_device_type_group(
    ansible_group_factory, device_type_factory, host_factory
):
    device_type1 = device_type_factory(name="type1")
    device_type2 = device_type_factory(name="type2")
    host1_t1 = host_factory(name="host1", device_type=device_type1)
    host2_t1 = host_factory(name="host2", device_type=device_type1)
    host1_t2 = host_factory(device_type=device_type2)
    group_t1 = ansible_group_factory(
        name="type1", type=models.AnsibleGroupType.DEVICE_TYPE
    )
    group_t2 = ansible_group_factory(
        name="type2", type=models.AnsibleGroupType.DEVICE_TYPE
    )
    group_t3 = ansible_group_factory(
        name="unknown", type=models.AnsibleGroupType.DEVICE_TYPE
    )
    assert group_t1.hosts == [host1_t1, host2_t1]
    assert group_t2.hosts == [host1_t2]
    assert group_t3.hosts == []


def test_ansible_dynamic_network_group(
    ansible_group_factory, network_factory, interface_factory, host_factory
):
    network1 = network_factory(vlan_name="network1")
    network2 = network_factory(vlan_name="network2")
    interface1_n1 = interface_factory(network=network1)
    interface2_n1 = interface_factory(network=network1)
    interface1_n2 = interface_factory(network=network2)
    host1_n1 = host_factory(name="host1", interfaces=[interface1_n1])
    host2_n1 = host_factory(name="host2", interfaces=[interface2_n1])
    host1_n2 = host_factory(interfaces=[interface1_n2])
    group_n1 = ansible_group_factory(
        name="network1", type=models.AnsibleGroupType.NETWORK
    )
    group_n2 = ansible_group_factory(
        name="network2", type=models.AnsibleGroupType.NETWORK
    )
    group_n3 = ansible_group_factory(
        name="unknown", type=models.AnsibleGroupType.NETWORK
    )
    assert group_n1.hosts == [host1_n1, host2_n1]
    assert group_n2.hosts == [host1_n2]
    assert group_n3.hosts == []


def test_ansible_dynamic_network_scope_group(
    ansible_group_factory,
    network_scope_factory,
    network_factory,
    interface_factory,
    host_factory,
):
    scope1 = network_scope_factory(name="scope1")
    scope2 = network_scope_factory(name="scope2")
    network1_s1 = network_factory(scope=scope1)
    network2_s1 = network_factory(scope=scope1)
    network1_s2 = network_factory(scope=scope2)
    interface1_s1 = interface_factory(network=network1_s1)
    interface2_s1 = interface_factory(network=network2_s1)
    interface1_s2 = interface_factory(network=network1_s2)
    host1_s1 = host_factory(name="host1", interfaces=[interface1_s1])
    host2_s1 = host_factory(name="host2", interfaces=[interface2_s1])
    host1_s2 = host_factory(interfaces=[interface1_s2])
    group_s1 = ansible_group_factory(
        name="scope1", type=models.AnsibleGroupType.NETWORK_SCOPE
    )
    group_s2 = ansible_group_factory(
        name="scope2", type=models.AnsibleGroupType.NETWORK_SCOPE
    )
    group_s3 = ansible_group_factory(
        name="unknown", type=models.AnsibleGroupType.NETWORK_SCOPE
    )
    assert group_s1.hosts == [host1_s1, host2_s1]
    assert group_s2.hosts == [host1_s2]
    assert group_s3.hosts == []
